package net.wanji.web.handler;

import lombok.extern.slf4j.Slf4j;
import net.wanji.feign.pojo.result.JsonViewObject;
import net.wanji.web.common.exception.CrossException;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import javax.servlet.http.HttpServletRequest;

import static net.wanji.web.common.enums.ResultEnum.INTERNAL_SERVER_ERROR;

/**
 * @author wuxiaokai
 * @date 2022/11/21 9:06:40
 */
@RestControllerAdvice
@Slf4j
public class GlobalExceptionHandler {

    /**
     * 处理自定义的业务异常
     */
    @ExceptionHandler(value = CrossException.class)
    public JsonViewObject exceptionHandler(HttpServletRequest req, CrossException e) {
        log.error("发生业务异常！原因是：{}", e.getErrorMsg());
        return JsonViewObject.newInstance().fail("发生业务异常！原因是：{}" + e.getErrorMsg());
    }

    /**
     * 处理空指针的异常
     */
    @ExceptionHandler(value = NullPointerException.class)
    public JsonViewObject exceptionHandler(HttpServletRequest req, NullPointerException e) {
        Throwable cause = e.getCause();
        String errorMessage;
        if (cause != null) {
            errorMessage = cause.getMessage();
        } else {
            StackTraceElement element = e.getStackTrace()[0];
            errorMessage = element.getClassName() + " " + element.getLineNumber();
        }
        log.error("发生空指针异常！位置在：{}", errorMessage);
        return JsonViewObject.newInstance().fail("发生空指针异常！位置在：" + errorMessage);
    }

    /**
     * 参数异常
     */
    @ExceptionHandler(value = MethodArgumentNotValidException.class)
    public JsonViewObject exceptionHandler(HttpServletRequest req, MethodArgumentNotValidException e) {
        FieldError fieldError = e.getBindingResult().getFieldError();
        String errorMessage = INTERNAL_SERVER_ERROR.getResultMsg();
        if (fieldError != null) {
            errorMessage = fieldError.getDefaultMessage();
        }
        log.error("发生参数异常！原因是：{}", errorMessage);
        return JsonViewObject.newInstance().fail("发生参数异常！原因是：" + errorMessage);
    }

    /**
     * 处理其他异常
     */
    @ExceptionHandler(value = Exception.class)
    public JsonViewObject exceptionHandler(HttpServletRequest req, Exception e) {
        Throwable cause = e.getCause();
        String errorMessage;
        if (cause != null) {
            errorMessage = cause.getMessage();
        } else {
            errorMessage = e.getMessage();
        }
        log.error("未知异常！原因是：{}", errorMessage);
        return JsonViewObject.newInstance().fail("未知异常！原因是：" + errorMessage);
    }
}
